@@ -17,11 +17,28 @@
 
 import abc
 
+from little_brother import dependency_injection
+from python_base_app.locale_helper import LocaleHelper
+
+_ = lambda x:x
+
+class RuleHandlerRunTimeException(RuntimeError):
+    pass
+
 class AbstractContextRuleHandler(object, metaclass=abc.ABCMeta):
 
-    def __init__(self, p_context_name):
+    def __init__(self, p_context_name, p_locale_helper=None):
 
         self._context_name = p_context_name
+        self._locale_helper = p_locale_helper
+
+    @property
+    def locale_helper(self) -> LocaleHelper:
+
+        if self._locale_helper is None:
+            self._locale_helper = dependency_injection.container[LocaleHelper]
+
+        return self._locale_helper
 
     @abc.abstractmethod
     def is_active(self, p_reference_date, p_details):
@@ -34,5 +51,18 @@ class AbstractContextRuleHandler(object, metaclass=abc.ABCMeta):
     def context_name(self):
         return self._context_name
 
+    def get_choices(self):
+        return []
+
+    def validate_context_details(self, p_context_detail):
+        pass # default action: none
+
+    def summary(self, p_context_detail):
+        if p_context_detail is not None and p_context_detail != "":
+            return [_("Details"), ": ", p_context_detail]
+
+        else:
+            return []
+
     def check_data(self):
-        pass
+        pass # default action: none
